import { EventEmitter, Directive, ElementRef, Renderer2, Input, HostListener, NgModule } from '@angular/core';
import { map, flatMap, takeUntil } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DragElementDirective {
    /**
     * @param {?} _elementRef
     * @param {?} _renderer
     */
    constructor(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        /**
         * \@description
         * Turn on/off drag option.
         *
         * \@usageNotes
         * 'x' axis for horizontal drag
         * 'y' axis for vertical drag
         * 'a' axis for all over document drag
         *  It's case sensitive (use lower case)
         *
         * \@default
         * true
         */
        this.dragElement = true;
        /**
         * \@description
         * Provide axis to drag.
         *
         * \@usageNotes
         * 'x' axis for horizontal drag
         * 'y' axis for vertical drag
         * 'a' axis for all over document drag
         *  It's case sensitive (use lower case)
         *
         * \@default
         * 'y'
         */
        this.dragAxis = "y";
        /**
         * \@description
         * Show drag icon or not.
         *
         * \@usageNotes
         * 'true' will show icon
         * 'false' will hide icon
         *
         * \@default
         * true
         */
        this.dragIcon = false;
        this._mouseDown = new EventEmitter();
        this._mouseMove = new EventEmitter();
        this._mouseup = new EventEmitter();
        this._mouseOut = new EventEmitter();
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    onMouseDown(event) {
        this._mouseDown.next(event);
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    onMouseMove(event) {
        this._mouseMove.next(event);
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    onMouseUp(event) {
        this._mouseup.next(event);
    }
    /**
     * @private
     * @param {?} event
     * @return {?}
     */
    onMouseOut(event) {
        this._mouseOut.next(event);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.dragElement) {
            this.setupEventSubscriber();
            this._mouseDragSubscribe = this._mouseDrag.subscribe((/**
             * @param {?} position
             * @return {?}
             */
            position => {
                // console.log(position);
                this.updateElementPosition(position);
            }));
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.dragElement) {
            this.setupData();
            this.setDragIcon();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._mouseDragSubscribe) {
            this._mouseDragSubscribe.unsubscribe();
        }
    }
    /**
     * @private
     * @return {?}
     */
    setupData() {
        if (this.dragAxis === "y") {
            this._dragPointerType = "n-resize";
        }
        else if (this.dragAxis === "x") {
            this._dragPointerType = "e-resize";
        }
        else if (this.dragAxis === "xy") {
            this._dragPointerType = "move";
        }
        else {
            this._dragPointerType = null;
        }
    }
    /**
     * @private
     * @return {?}
     */
    setupEventSubscriber() {
        this._mouseDrag = this._mouseDown.pipe(map((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            event.preventDefault();
            this._initialPosition = {
                left: this._elementRef.nativeElement.style.left,
                top: this._elementRef.nativeElement.style.top
            };
            return {
                left: event["clientX"] -
                    this._elementRef.nativeElement.getBoundingClientRect().left,
                top: event["clientY"] -
                    this._elementRef.nativeElement.getBoundingClientRect().top
            };
        })), flatMap((/**
         * @param {?} offset
         * @return {?}
         */
        offset => {
            return this._mouseMove.pipe(map((/**
             * @param {?} event
             * @return {?}
             */
            (event) => {
                return {
                    left: event["clientX"] - offset.left,
                    top: event["clientY"] - offset.top
                };
            })), takeUntil(this._mouseup), takeUntil(this._mouseOut));
        })));
    }
    /**
     * @private
     * @return {?}
     */
    setDragIcon() {
        if (this.dragIcon) {
            /** @type {?} */
            const div = document.createElement("div");
            div.style.textAlign = "center";
            div.style.fontSize = " 1.5em";
            div.style.padding = "0.2em 0";
            div.style.color = "#1565c0";
            div.className = "drag-icon-wrapper";
            div.innerHTML = this.dragIconInnerText;
            this._renderer.appendChild(this._elementRef.nativeElement, div);
        }
        if (this._dragPointerType) {
            this._renderer.setStyle(this._elementRef.nativeElement, "cursor", this._dragPointerType);
        }
        if (this.dragElementClass) {
            this._renderer.addClass(this._elementRef.nativeElement, this.dragElementClass);
        }
    }
    /**
     * @private
     * @param {?} position
     * @return {?}
     */
    updateElementPosition(position) {
        if (!this._dragPointerType) {
            return;
        }
        if (this.dragAxis === "x") {
            this._renderer.setStyle(this._elementRef.nativeElement, "left", position.left + "px");
            this._renderer.setStyle(this._elementRef.nativeElement, "top", this._initialPosition.top + "px");
        }
        else if (this.dragAxis === "y") {
            this._renderer.setStyle(this._elementRef.nativeElement, "left", this._initialPosition.left + "px");
            this._renderer.setStyle(this._elementRef.nativeElement, "top", position.top + "px");
        }
        else if (this.dragAxis === "xy") {
            this._renderer.setStyle(this._elementRef.nativeElement, "top", position.top + "px");
            this._renderer.setStyle(this._elementRef.nativeElement, "left", position.left + "px");
        }
    }
}
DragElementDirective.decorators = [
    { type: Directive, args: [{
                selector: "[ngx-drag-element]"
            },] }
];
/** @nocollapse */
DragElementDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
DragElementDirective.propDecorators = {
    dragElement: [{ type: Input, args: ["ngx-drag-element",] }],
    dragElementClass: [{ type: Input }],
    dragAxis: [{ type: Input }],
    dragIcon: [{ type: Input }],
    dragIconInnerText: [{ type: Input }],
    onMouseDown: [{ type: HostListener, args: ["mousedown", ["$event"],] }],
    onMouseMove: [{ type: HostListener, args: ["mousemove", ["$event"],] }],
    onMouseUp: [{ type: HostListener, args: ["mouseup", ["$event"],] }],
    onMouseOut: [{ type: HostListener, args: ["mouseout", ["$event"],] }]
};
if (false) {
    /**
     * \@description
     * Turn on/off drag option.
     *
     * \@usageNotes
     * 'x' axis for horizontal drag
     * 'y' axis for vertical drag
     * 'a' axis for all over document drag
     *  It's case sensitive (use lower case)
     *
     * \@default
     * true
     * @type {?}
     */
    DragElementDirective.prototype.dragElement;
    /**
     * \@description
     * Set style for drag element
     *
     * \@usageNotes
     * All css style can be applied
     *
     * \@default
     * undefined
     * @type {?}
     */
    DragElementDirective.prototype.dragElementClass;
    /**
     * \@description
     * Provide axis to drag.
     *
     * \@usageNotes
     * 'x' axis for horizontal drag
     * 'y' axis for vertical drag
     * 'a' axis for all over document drag
     *  It's case sensitive (use lower case)
     *
     * \@default
     * 'y'
     * @type {?}
     */
    DragElementDirective.prototype.dragAxis;
    /**
     * \@description
     * Show drag icon or not.
     *
     * \@usageNotes
     * 'true' will show icon
     * 'false' will hide icon
     *
     * \@default
     * true
     * @type {?}
     */
    DragElementDirective.prototype.dragIcon;
    /**
     * \@description
     * Specify drag icon element as innerText.
     *
     * \@usageNotes
     * Give icon element as innerText in "string" value
     * \@example '<i class="fa fa-arrows-v"></i>'
     *
     * \@default
     * undefined
     * @type {?}
     */
    DragElementDirective.prototype.dragIconInnerText;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._dragPointerType;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._initialPosition;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._mouseDown;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._mouseMove;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._mouseup;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._mouseOut;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._mouseDragSubscribe;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._mouseDrag;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._elementRef;
    /**
     * @type {?}
     * @private
     */
    DragElementDirective.prototype._renderer;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DragElementModule {
}
DragElementModule.decorators = [
    { type: NgModule, args: [{
                declarations: [DragElementDirective],
                imports: [],
                exports: [DragElementDirective]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { DragElementDirective, DragElementModule };
//# sourceMappingURL=angular-drag-element.js.map
